#!/usr/bin/env python
##
# Copyright (c) 2000 by g.p.ciceri <gp.ciceri@acm.org>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# Alternatively, you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#                                   
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU (Library) General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.                    
#
#
# $Author: zanzi $
# $Revision: 1.1 $
# $Log: Futures.py,v $
# Revision 1.1  2000/01/02 17:44:29  zanzi
# Initial revision
#
#
# module name: Futures.py
#
# description: python functions to handle futures prices algorithms
#
#

from math import exp, sqrt, log
from NormDist import N

def FuturesPrice(underlyingPrice, interestRate, time):
    c = exp(interestRate * time) * underlyingPrice
    return c

def FuturesOptionPriceCallEuropeanBlack(futuresPrice, strikePrice, interestRate, sigma, time):
    sigmaSqr = sigma * sigma
    timeSqrt = sqrt(time)
    d1 = (log(futuresPrice/strikePrice) + 0.5 * sigmaSqr * time) / (sigma * timeSqrt)
    d2 = d1 - sigma * timeSqrt
    return (exp(- interestRate * time) * (futuresPrice * N(d1) - strikePrice * N(d2)))

def FuturesOptionPriceCallAmericanBinomial(futuresPrice, strikePrice, interestRate, sigma, time, stepsNum):
    fxPrices = []
    callValues = []
    tDelta = time/stepsNum
    Rinv = exp(-r * tDelta)
    u = exp(sigma * sqrt(tDelta))
    d = 1.0/u
    uu = u * u
    pUp = (1 - d)/(u - d)
    pDown = 1.0 - pUp
    fxPrices.append(futuresPrice * pow(d, stepsNum))
    for i in xrange(1, stepsNum+1):
        fxPrices.append(uu * fxPrices[i-1])
    for i in xrange(stepsNum+1):
        callValues.append(max(0.0,fxPrices[i]-strikePrice))
    for step in xrange(stepsNum, -1, -1):
        for i in xrange(step):
            fxPrices[i] = d * fxPrices[i+1]
            callValues[i] = (pDown * callValues[i] + pUp * callValues[i+1]) * Rinv
            callValues[i] = max(callValues[i], fxPrices[i] - strikePrice)
    return callValues[0]

 


if __name__ == '__main__':
    S = 100
    F = 100
    X = 100
    r = 0.1
    sigma = 0.2
    timeToMaturity = 1.0
    print 'futures price ', FuturesPrice(S,r,timeToMaturity)
    print 'european call price ', FuturesOptionPriceCallEuropeanBlack(F, X, r, sigma, timeToMaturity)
    nosteps = 100
    print 'call price binomial american ', FuturesOptionPriceCallAmericanBinomial(F, X, r, sigma, timeToMaturity, nosteps)





