#!/usr/bin/env python
##
# Copyright (c) 2000 by g.p.ciceri <gp.ciceri@acm.org>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# Alternatively, you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#                                   
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU (Library) General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# $Author: zanzi $
# $Revision: 1.1 $
# $Log: NormDist.py,v $
# Revision 1.1  2000/01/02 17:43:08  zanzi
# Initial revision
#
#
# module name: NormDist.py
#
# description: python functions to handle normal distributions
#
#

from types import ListType, TupleType
from math import exp, sqrt

PI = 3.141592653589793238462643

    
def n(z):
    return (1.0/sqrt(2.0*PI))*exp(-0.5*z*z)

def nMuSigma(r, mu, sigma):
    nv = 1.0/(sqrt(2.0*PI)*sigma)
    z = (r - mu)/sigma
    nv = nv * exp(-0.5*z*z)
    return nv

def N(z):
    b1 = 0.31938153
    b2 = -0.356563782
    b3 = 1.781477937
    b4 = -1.821255978
    b5 = 1.330274429
    p  = 0.2316419
    c2 = 0.3989423
    if (z > 6.0): return 1.0
    if (z < -6.0): return 0.0
    a = abs(z)
    t = 1.0/(1.0+a*p)
    b = c2 * exp(-z * (z/2.0))
    n = ((((b5 * t + b4) * t + b3) * t + b2) * t + b1) * t
    n = 1.0 - b * n
    if (z < 0.0): n = 1.0 - n
    return n

def f(x, y, aPrime, bPrime, rho):
    r = aPrime*(2*x-aPrime) + bPrime*(2*y-bPrime) + 2 * rho * (x - aPrime) * (y - bPrime)
    return exp(r)

def sgn(x):
    if (x >= 0.0): return 1.0
    else: return -1.0

def NRho(a, b, rho):
    if ( (a <= 0.0) and (b <= 0.0) and (rho <= 0.0) ):
        aPrime = a / sqrt(2.0 * (1.0 - rho * rho))
        bPrime = b / sqrt(2.0 * (1.0 - rho * rho))
        A = [0.3253030, 0.4211071, 0.1334425, 0.006374323]
        B = [0.1337764, 0.6243247, 1.3425378, 2.2626645]
        sum = 0
        for i in xrange(4):
            for j in xrange(4):
                sum = sum + A[i] * A[j] * f(B[i],B[j],aPrime,bPrime,rho) 
        sum = sum * (sqrt(1.0 - rho * rho)/PI)
        return sum
    elif ( (a * b * rho) <= 0.0 ):
        if ( (a <= 0.0) and (b >= 0.0) and (rho >= 0.0) ):
            return (N(a) - NRho(a, -b, -rho))
        elif ( (a >= 0.0) and (b <= 0.0) and (rho >= 0.0) ):
            return (N(b) - NRho(-a, b, -rho))
        elif ( (a >= 0.0) and (b >= 0.0) and (rho <= 0.0) ):
            return (N(a) + N(b) - 1.0 + NRho(-a, -b, rho))
    elif ( (a * b * rho) >= 0.0 ):
        denum = sqrt( a*a - 2*rho*a*b + b*b)
        rho1  = ((rho * a - b) * sgn(a))/denum
        rho2  = ((rho * b - a) * sgn(b))/denum 
        delta = (1.0-sgn(a)*sgn(b))/4.0
        return (NRho(a, 0.0, rho1) + NRho(b, 0.0, rho2) - delta)
    return none
        

    
if __name__ == '__main__': 
    print n(0.5)
    print N(0.5)
 
